%%%-------------------------------------------------------------------
%%% @uthor Dave Kuhlman
%%% @copyright (C) 2017, Dave Kuhlman
%%% @doc
%%%
%%% @end
%%% Created : 2017-04-04 14:18:06.971693
%%%-------------------------------------------------------------------
-module(test07a_handler).

-export([
         init/2,
         allowed_methods/2,
         content_types_provided/2,
         content_types_accepted/2,
         delete_resource/2
         %delete_completed/2
        ]).

-export([
         to_json/2,
         from_json/2
        ]).

-record(state, {op, response}).

%%%-------------------------------------------------------------------
%%% Cowby ReST built-in callbacks
%%%-------------------------------------------------------------------

init(Req, Opts) ->
    %io:fwrite("(init) Opts: ~p~n", [Opts]),
    [{op, Op} | _] = Opts,
    State = #state{op=Op, response=none},
    {cowboy_rest, Req, State}.

%%allowed_methods(Req, State) ->
%%    Value = [<<"GET">>, <<"HEAD">>, <<"OPTIONS">>],
%%    {Value, Req, State}.

allowed_methods(Req, State) ->
    %Value = [<<"GET">>, <<"POST">>, <<"PUT">>, <<"DELETE">>],
    Value = [<<"GET">>, <<"POST">>, <<"DELETE">>],
    {Value, Req, State}.

%%allow_missing_post(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%charsets_provided(Req, State) ->
%%    Value = skip,
%%    {Value, Req, State}.

%%content_types_accepted(Req, State) ->
%%    Value = none,
%%    {Value, Req, State}.

content_types_accepted(Req, State) ->
    Value = [
             %{{ <<"text">>, <<"json">>, '*'}, from_json},
             {<<"application/x-www-form-urlencoded">>, from_json}
            ],
    {Value, Req, State}.

%%content_types_provided(Req, State) ->
%%    Value = [{{ <<"text">>, <<"html">>, '*'}, to_html}],
%%    {Value, Req, State}.

content_types_provided(Req, State) ->
    Value = [
             {{ <<"application">>, <<"json">>, '*'}, to_json}
            ],
    {Value, Req, State}.

%%delete_completed(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%~ delete_completed(Req, #state{response=Response} = State) ->
%~     Response,
%~     %io:fwrite("(delete_completed) Response: ~p~n", [Response]),
%~     %io:fwrite("-----\n(delete_completed) Req: ~p~n-----~n", [Req]),
%~     %Req1 = cowboy_req:reply(200, #{
%~     %      <<"content-type">> => <<"application/json">>
%~     %}, Response, Req),
%~     %io:fwrite("-----\n(delete_completed) Req1: ~p~n-----~n", [Req1]),
%~     {true, Req, State}.

%%delete_resource(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

delete_resource(Req, State) ->
    RecordId = cowboy_req:binding(doc_id, Req),
    Result = case db_server:delete_rec(RecordId) of
        ok ->
            Body = {[
                     {<<"code">>, <<"deleted">>},
                     {<<"recordid">>, RecordId}
                     ]},
            Body1 = jiffy:encode(Body),
            {true, Req, State#state{response=Body1}};
        {error, Reason} ->
            io:fwrite("(delete_resource) error Reason: ~p~n", [Reason]),
            Body = {[
                     {<<"code">>, <<"error">>},
                     {<<"recordid">>, RecordId},
                     {<<"reason">>, Reason}
                     ]},
            Body1 = jiffy:encode(Body),
            {false, Req, State#state{response=Body1}}
    end,
%~     Value = false,
%~     {Value, Req, State}.
    Result.

%%expires(Req, State) ->
%%    Value = undefined,
%%    {Value, Req, State}.

%%forbidden(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%generate_etag(Req, State) ->
%%    Value = undefined,
%%    {Value, Req, State}.

%%is_authorized(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%is_conflict(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%known_methods(Req, State) ->
%%    Value = [<<"GET">>, <<"HEAD">>, <<"POST">>, <<"PUT">>, <<"PATCH">>, <<"DELETE">>, <<"OPTIONS">>],
%%    {Value, Req, State}.

%%languages_provided(Req, State) ->
%%    Value = skip,
%%    {Value, Req, State}.

%%last_modified(Req, State) ->
%%    Value = undefined,
%%    {Value, Req, State}.

%%malformed_request(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%moved_permanently(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%moved_temporarily(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%multiple_choices(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%options(Req, State) ->
%%    Value = ok,
%%    {Value, Req, State}.

%%previously_existed(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%resource_exists(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%service_available(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%uri_too_long(Req, State) ->
%%    Value = false,
%%    {Value, Req, State}.

%%valid_content_headers(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%valid_entity_length(Req, State) ->
%%    Value = true,
%%    {Value, Req, State}.

%%variances(Req, State) ->
%%    Value = [],
%%    {Value, Req, State}.

%%%-------------------------------------------------------------------
%%% Additional callbacks
%%%-------------------------------------------------------------------

to_json(Req, #state{op=Op} = State) ->
    %io:fwrite("~n-----~n(to_json) Req: ~p~n-----~n", [Req]),
    Result = case Op of
        get ->
            to_json_get(Req, State);
        list ->
            to_json_list(Req, State);
        help ->
            to_json_help(Req, State)
    end,
    %io:fwrite("~n-----~nResult: ~p~n-----~n", [Result]),
    Result.

from_json(Req, #state{op=Op} = State) ->
    %Method = cowboy_req:method(Req),
    %io:fwrite("(from_json) Method: ~p~n", [Method]),
    Result = case Op of
                 create ->
                     %io:fwrite("(from_json) create~n", []),
                     Reply = cowboy_req:read_urlencoded_body(Req),
                     %io:fwrite("(from_json) Reply: ~p~n", [Reply]),
                     {ok, [{<<"content">>, Content}], Req1} = Reply,
                     {ok, DocId} = db_server:create_rec(Content),
                     Body = {[
                              {<<"code">>, <<"created">>},
                              {<<"resource_id">>, DocId}
                             ]},
                     Body1 = jiffy:encode(Body),
                     %io:fwrite("*** (from_json) Body1: ~p~n", [Body1]),
                     {{true, Body1}, Req1, State};
                 update ->
                     %io:fwrite("(from_json) update~n", []),
                     RecordId = cowboy_req:binding(doc_id, Req),
                     Reply = cowboy_req:read_urlencoded_body(Req),
                     %io:fwrite("(from_json) Reply: ~p~n", [Reply]),
                     {ok, [{<<"content">>, Content}], Req1} = Reply,
                     %io:fwrite("(from_json) PUT RecordId: ~p  Content: ~p~n",
                     %          [RecordId, Content]),
                     case db_server:rec_exists(RecordId) of
                         ok ->
                             Response = db_server:update_rec(RecordId, Content),
                             case Response of
                                 {ok, _} ->
                                     Body = {[
                                              {<<"code">>, <<"updated">>},
                                              {<<"id">>, RecordId},
                                              {<<"record">>, Content}
                                             ]},
                                     Body1 = jiffy:encode(Body),
                                     Req2 = cowboy_req:reply(200, #{
                                         <<"content-type">> => <<"application/json">>
                                     }, Body1, Req1),
                                     %io:fwrite("(from_json) PUT Body1: ~p~n",
                                     %          [Body1]),
                                     {{true, RecordId}, Req2, State};
                                 {error, not_found} ->
                                     Reason = list_to_binary(
                                                io_lib:format(
                                                  "record ID ~p not found",
                                                  [RecordId])),
                                     Body = {[
                                              {<<"code">>, <<"not_found">>},
                                              {<<"reason">>, Reason}
                                             ]},
                                     {jiffy:encode(Body), Req1, State};
                                 {error, too_many_records} ->
                                     Reason = list_to_binary(
                                                io_lib:format(
                                                  "extra records for ID ~p",
                                                  [RecordId])),
                                     Body = {[
                                              {<<"code">>, <<"extra_records">>},
                                              {<<"reason">>, Reason}
                                             ]},
                                     {jiffy:encode(Body), Req1, State}
                             end;
                         not_found ->
                             Reason = list_to_binary(io_lib:format(
                                                       "record ID ~p not found",
                                                       [RecordId])),
                             Body = {[
                                      {<<"code">>, not_found},
                                      {<<"reason">>, Reason}
                                     ]},
                             {jiffy:encode(Body), Req1, State}
                     end
             end,
    %io:fwrite("(from_json) Result: ~p~n", [Result]),
    Result.

%%%-------------------------------------------------------------------
%%% Private functions
%%%-------------------------------------------------------------------

to_json_get(Req, State) ->
    RecordId = cowboy_req:binding(doc_id, Req),
    RecordId1 = RecordId,
    %io:fwrite("(to_json_get) RecordId: ~p~n", [RecordId]),
    Response = db_server:get_rec(RecordId),
    Body = case Response of
        {ok, Data} ->
            {[
              {<<"id">>, RecordId1},
              {<<"record">>, Data}
             ]};
        {error, not_found} ->
            Reason = list_to_binary(
                io_lib:format("record ~p not found", [RecordId1])),
            {[
              {<<"code">>, <<"not_found">>},
              {<<"reason">>, Reason}
             ]};
        {error, too_many_records} ->
            Reason = list_to_binary(
                io_lib:format("extra records for ~p", [RecordId1])),
            {[
              {<<"code">>, <<"not_found">>},
              {<<"reason">>, Reason}
             ]}
           end,
    Body1 = jiffy:encode(Body),
    %io:fwrite("(to_json_get) Body1: ~p~n", [Body1]),
    {Body1, Req, State}.

to_json_list(Req, State) ->
    {ok, Records} = db_server:get_all_recs(),
    %io:fwrite("(to_json_list) Records: ~p~n", [Records]),
    Body = {[{records, Records}]},
    %io:fwrite("(to_json_list) Body: ~p~n", [Body]),
    {jiffy:encode(Body), Req, State}.

to_json_help(Req, State) ->
    Body = {[
         {<<"version">>, <<"4">>},
         {<<"app">>, <<"test07a">>},
         {<<"/docs/get/<doc_id>">>, <<"retrieve document by ID">>},
         {<<"/docs/list">>, <<"retrieve all documents">>},
         {<<"/docs/create">>, <<"create document. body is url encoded form data">>},
         {<<"/docs/update">>, <<"update document. body is url encoded form data">>},
         {<<"/docs/delete">>, <<"delete document">>},
         {<<"/docs/help">>, <<"print this help">>}
         ]},
    {jiffy:encode(Body), Req, State}.
